#include "extension.h"

SMFFMPEG g_SMFFMPEG;

SMEXT_LINK(&g_SMFFMPEG);

std::unordered_map<std::string, cell_t> SoundDurationCache;

cell_t GetSoundDuration(IPluginContext *pContext, const cell_t *params)
{
	cell_t length = 0;
	char *file;
	struct stat s;

	pContext->LocalToString(params[1], &file);

	if (SoundDurationCache.count(file))
		return SoundDurationCache[file];

	if (access(file, R_OK) != 0) return 0; // No read access, or file doesn't exist
	if (stat(file, &s) != 0) return 0; // Can't stat it, so why bother? Though access should be calling stat itself anyway...
	if (s.st_mode & S_IFDIR) return 0; // It's a directory you dunce
	if (!(s.st_mode & (S_IFREG | S_IFLNK))) return 0; // It's not a standard file or symlink
	// (S_IFBLK | S_IFCHR | S_IFIFO) files might work fine, but would anyone be using these?

	AVFormatContext* pFormatCtx = avformat_alloc_context();
	if(!pFormatCtx)
		return 0;

	if (avformat_open_input(&pFormatCtx, file, NULL, NULL) < 0) {
		avformat_close_input(&pFormatCtx);
		avformat_free_context(pFormatCtx);
		SoundDurationCache[file] = 0;
		return 0;
	}

	if (avformat_find_stream_info(pFormatCtx, NULL) < 0) {
		avformat_close_input(&pFormatCtx);
		avformat_free_context(pFormatCtx);
		SoundDurationCache[file] = 0;
		return 0;
	}

	length = pFormatCtx->duration / AV_TIME_BASE;

	if (length < 0) {
 		// Error code, garbled data, or integer overflow
		avformat_close_input(&pFormatCtx);
		avformat_free_context(pFormatCtx);
		SoundDurationCache[file] = 0;
		return 0;
	}

	avformat_close_input(&pFormatCtx);
	avformat_free_context(pFormatCtx);

	SoundDurationCache[file] = length;
	return length;
}

cell_t FlushSoundDurationCache(IPluginContext *pContext, const cell_t *params)
{
	SoundDurationCache.erase(SoundDurationCache.begin(), SoundDurationCache.end());

	return 0;
}

const sp_nativeinfo_t MyNatives[] =
{
	{"SMFFMPEG_GetSoundDuration",		GetSoundDuration},
	{"SMFFMPEG_FlushSoundDurationCache", 	FlushSoundDurationCache},
	{NULL,					NULL},
};

void SMFFMPEG::SDK_OnAllLoaded()
{
	av_register_all();
	sharesys->AddNatives(myself, MyNatives);
}

bool SMFFMPEG::SDK_OnLoad(char *error, size_t maxlength, bool late)
{
	sharesys->RegisterLibrary(myself, "smffmpeg");
	return true;
}
